﻿using System;
using System.Reflection;
using UnityEditor;
using UnityEditor.Search;
using UnityEditor.Experimental.GraphView;
using System.Collections.Generic;
using System.Linq;
using ImageProcessingGraph.Editor.Nodes.NodeAttributes;
using Object = UnityEngine.Object;
static class IPT_SearchProvider
{
    internal static string id = "IPT_Tree";
    internal static List<SearchContextElement> elements;

    [SearchItemProvider]
    internal static SearchProvider CreateProvider()
    {
        return new SearchProvider(id, "IPT Nodes")
        {
            filterId = "tree:",
            priority = 99999,
            showDetailsOptions = ShowDetailsOptions.Inspector | ShowDetailsOptions.Actions,
            fetchItems = (context, items, provider) => FetchItems(context, provider),
            fetchLabel = (item, context) => item.label,
            fetchDescription = (item, context) => item.description,
            toObject = (item, type) => item.data as Object
        };
    }

    private static IEnumerable<SearchItem> FetchItems(SearchContext context, SearchProvider provider)
    {
        if (elements == null)
            BuildElements();

        foreach (var element in elements)
        {
            if (!element.title.ToLower().Contains(context.searchQuery.ToLower()))
                continue;

            // var item = provider.CreateItem(context, element.title, element.title, $"Node: {element.title}", 0, element.target);
            var item = provider.CreateItem(context, element.title, element.title, element.title, null, element.target);
            yield return item;
        }
    }

    private static void BuildElements()
    {
        elements = new List<SearchContextElement>();

        foreach (var type in TypeCache.GetTypesWithAttribute<NodeInfoAttribute>())
        {
            var attr = type.GetCustomAttribute<NodeInfoAttribute>();
            if (string.IsNullOrEmpty(attr.MenuItem))
                continue;

            var node = Activator.CreateInstance(type);
            elements.Add(new SearchContextElement(node, attr.MenuItem));
        }

        elements.Sort((entry1, entry2) =>
        {
            string[] splits1 = entry1.title.Split('/');
            string[] splits2 = entry2.title.Split('/');

            for (int i = 0; i < splits1.Length; i++)
            {
                if (i >= splits2.Length) return 1;
                int value = splits1[i].CompareTo(splits2[i]);
                if (value != 0)
                {
                    if (splits1.Length != splits2.Length && (i == splits1.Length - 1 || i == splits2.Length - 1))
                        return splits1.Length < splits2.Length ? 1 : -1;

                    return value;
                }
            }
            return 0;
        });
    }
}

public class SearchContextElement
{
    public object target;
    public string title;

    public SearchContextElement(object target, string title)
    {
        this.target = target;
        this.title = title;
    }
}
